from django.contrib.auth.models import User
from django.db import IntegrityError
from django.test import TestCase

from feedme.models import Feed, FeedItem


class FeedMeTestCase(TestCase):

    def test_update_feed(self):
        feed = Feed(url='http://rss.cnn.com/rss/cnn_topstories.rss')
        feed.save()
        feed.update(force=True)
        self.assertEqual(FeedItem.objects.all().count(), 10)

    def test_mark_as_read(self):
        feed = Feed(url='http://rss.cnn.com/rss/cnn_topstories.rss')
        feed.save()
        feed.update(force=True)
        self.assertEqual(FeedItem.objects.un_read().count(), 10)
        for item in FeedItem.objects.un_read():
            item.mark_as_read()
        self.assertEqual(FeedItem.objects.un_read().count(), 0)

    def test_me(self):
        self.assertTrue(True, True)

    def test_multiple_users_same_feed(self):
        user1 = User(username="user1")
        user1.set_unusable_password()
        user2 = User(username="user2")
        user2.set_unusable_password()

        user1.save()
        user2.save()

        feed_url = 'http://rss.cnn.com/rss/cnn_topstories.rss'
        feed1 = Feed(user=user1, url=feed_url)
        feed2 = Feed(user=user2, url=feed_url)
        try:
            feed1.save()
            feed2.save()
        except IntegrityError:
            self.fail("Couldn't add %s for both %s and %s" % (feed_url, user1, user2))

        self.assertIsNotNone(feed1.pk)
        self.assertIsNotNone(feed2.pk)




# -*- coding: utf-8 -*-

import os
import shutil
import zipfile

from tempfile import mktemp
from lxml import etree
from json import loads


class GoogleReaderTakeout(object):

    def __init__(self, file_obj):
        self.directory = ""
        self.temp = mktemp()
        self._unpack(file_obj)

    def _unpack(self, file_obj):
        z = zipfile.ZipFile(file_obj)
        z.extractall(path=self.temp)
        for (path, dirs, files) in os.walk(self.temp):
            if 'subscriptions.xml' in files:
                self.directory = path
        z.close()

    def _load_xml_file(self, filename):
        subscriptions = '%s/%s.xml' % (self.directory, filename)
        return etree.XML(''.join(open(subscriptions).readlines()))

    def _load_json_file(self, filename):
        json_file = '%s/%s.json' % (self.directory, filename)
        return loads(''.join(open(json_file).readlines()))

    def subscriptions(self):
        grab = self._load_xml_file('subscriptions')
        data = []
        for element in grab.xpath('//opml/body/outline'):
            if element.get('type') is None:
                # if type is none it's a category
                for child in element.getchildren():
                    data.append({
                        'text': child.get('text'),
                        'title': child.get('title'),
                        'type': child.get('type'),
                        'xmlUrl': child.get('xmlUrl'),
                        'htmlUrl': child.get('htmlUrl'),
                        'category': element.get('title'),
                    })
            else:
                data.append({
                    'text': element.get('text'),
                    'title': element.get('title'),
                    'type': element.get('type'),
                    'xmlUrl': element.get('xmlUrl'),
                    'htmlUrl': element.get('htmlUrl'),
                    'category': None,
                })
        return data

    def __getattr__(self):
        def method(*args, **kwargs):
            getattr(self, '_load_json_file')(*args, **kwargs)
        return method

    def __del__(self):
        shutil.rmtree(self.temp)



"""
Django Feedme

Mixins.py

Author: Derek Stegelman
"""
import json

from django.http import HttpResponse


class AjaxableResponseMixin(object):
    """
    Mixin to add AJAX support to a form.
    Must be used with an object-based FormView (e.g. CreateView)
    """

    def render_to_json_response(self, context, **response_kwargs):
        """
        Render the output as JSON
        :param context:
        :param response_kwargs:
        :return:
        """
        data = json.dumps(context)
        response_kwargs['content_type'] = 'application/json'
        return HttpResponse(data, **response_kwargs)

    def form_invalid(self, form):
        """
        If the request is made via AJAX return as such.
        :param form:
        :return:
        """
        response = super(AjaxableResponseMixin, self).form_invalid(form)
        if self.request.is_ajax():
            return self.render_to_json_response(form.errors, status=400)
        else:
            return response

    def form_valid(self, form):
        """
        We make sure to call the parent's form_valid() method because
        it might do some processing (in the case of CreateView, it will
        call form.save() for example).
        """
        response = super(AjaxableResponseMixin, self).form_valid(form)
        if self.request.is_ajax():
            data = {
                'pk': self.object.pk,
            }
            return self.render_to_json_response(data)
        else:
            return response



from celery.task import task


@task(name='update_feeds')
def update_feed(feed):
    feed._update_feed()


@task(name='update_all_feeds')
def update_all_feeds():
    from .models import Feed
    for feed in Feed.objects.all():
        feed._update_feed()


@task(name='send_digest')
def send_digest():
    import digest
    digest.send_digest()